#ifndef _DW_FILTER_C
#define _DW_FILTER_C

#ifndef lint
static const char rcsid[] = "$Id: dw_filter.c,v 1.18 2001/06/18 06:02:32 cvser Exp $";
#endif

#include <unistd.h>
#include <stdio.h>
#include <syslog.h>
#include <string.h>
#include "dwc_lib.h"
#include "dwc_scan.h"
#include "dwc_macro.h"
#include "dw_filter.h"

typedef SMFICTX         DWF_context_t;
typedef sfsistat        DWF_retcode_t;

static DWF_retcode_t dwf_connect ( DWF_context_t *context, char* peername, _SOCK_ADDR *peeraddr );
/* static DWF_retcode_t dwf_helo ( DWF_context_t *context, char* helo ); */
static DWF_retcode_t dwf_from ( DWF_context_t *context, char* from[] );
static DWF_retcode_t dwf_rcpt ( DWF_context_t *context, char* rcpt[] );
static DWF_retcode_t dwf_header ( DWF_context_t *context, char* hdr_name, char* hdr_value );
static DWF_retcode_t dwf_header_end ( DWF_context_t *context );
static DWF_retcode_t dwf_msg_chunk ( DWF_context_t *context, u_char* chunk, size_t length );  
static DWF_retcode_t dwf_msg_end ( DWF_context_t *context );
static DWF_retcode_t dwf_abort ( DWF_context_t *context );
static DWF_retcode_t dwf_close ( DWF_context_t *context );
 
struct smfiDesc drweb_filter_desc =
{
	"DrWeb Sendmail Filter v1.2", /* filter name */
	SMFI_VERSION,	/* version code -- do not change */
	SMFIF_ADDHDRS | SMFIF_CHGBODY | SMFIF_ADDRCPT | SMFIF_CHGHDRS,	/* flags */
	dwf_connect,    /* connection info filter */
	NULL,     		/* SMTP HELO command filter */
	dwf_from,	    /* envelope sender filter */
	dwf_rcpt,		/* envelope recipient filter */
	dwf_header,	    /* header filter */
	dwf_header_end,	/* end of header */
	dwf_msg_chunk,	/* body block filter */
	dwf_msg_end,	/* end of message */
	dwf_abort,	    /* message aborted */
	dwf_close       /* connection cleanup */
};

char  ArcDir[ PATH_MAX + 1 ] = { CHAR0 };
char  ShareDir[ PATH_MAX + 1 ] = { CHAR0 };
char  ArcTemplate[ PATH_MAX + 1 ] = { CHAR0 };
char  ShareTemplate[ PATH_MAX + 1 ] = { CHAR0 };
char  PostmasterMail[ PATH_MAX + 1 ] = { CHAR0 };
char  NotifierMail[ PATH_MAX + 1 ] = { CHAR0 };
int   scanOptions = 0;
int   bLocal = 0;
int   bQuiet = 0;
int   bPostmasterNotify = 0;
int   bSenderNotify = 0;
sfsistat ActionOnError = SMFIS_REJECT;

typedef struct {
    int           fd_tmp;   /* fd for temp file */
    FILE         *fs_tmp;   /* file stream for temp file */
    char         *fname_tmp;/* name of temp file */
    char         *from;     /* host that send this message */
    int           contents; /* number of headers "Content-Type" */
    int           from_num; /* number of headers "From" */
} DWF_private_data_t;

/* ---------------------------------------------------------------------- */
#define VERBOSE( expr )    if( bQuiet ){ (expr); }

/* ---------------------------------------------------------------------- */
#define get_private_data( ctx )        ((DWF_private_data_t*) smfi_getpriv((ctx)))

/* ---------------------------------------------------------------------- */
#define set_private_data( ctx, priv )  smfi_setpriv((ctx), (priv))

/* ---------------------------------------------------------------------- */
#define create_private_data( context, priv )                                \
BEGIN_MACRO                                                                 \
    (priv) = DWC_Malloc( DWF_private_data_t*, sizeof(DWF_private_data_t) ); \
    if( !(priv) )                                                           \
    {                                                                       \
        syslog( LOG_MAIL | LOG_DEBUG, "can't create private data from context" );\
                    /* it's not bug, where priv == NULL */                  \
        FAIL_EXIT( (context), (priv) );                                     \
    }                                                                       \
    (priv)->fd_tmp = -1;                                                    \
    (priv)->fs_tmp = NULL;                                                  \
    (priv)->from = NULL;                                                    \
    (priv)->contents = 0;                                                   \
    (priv)->from_num = 0;                                                   \
END_MACRO

/* ---------------------------------------------------------------------- */
#define free_private_data( priv )               \
BEGIN_MACRO                                     \
    if( (priv)->fs_tmp )                        \
    {                                           \
        fflush( (priv)->fs_tmp );               \
        fclose( (priv)->fs_tmp );               \
        (priv)->fd_tmp = -1;                    \
    }                                           \
    if( (priv)->fd_tmp >= 0 )                   \
    {                                           \
        close( (priv)->fd_tmp );                \
    }                                           \
    if( (priv)->fname_tmp )                     \
    {                                           \
        unlink( (priv)->fname_tmp );            \
        /* ASV: allocated by strdup */          \
        free( (priv)->fname_tmp );              \
    }                                           \
    if( (priv)->from )                          \
    {   /* ASV: allocated by strdup */          \
        free( (priv)->from );                   \
    }                                           \
         /* after next priv == NULL */          \
    DWC_Free( (priv) );                         \
END_MACRO

/* ---------------------------------------------------------------------- */
#define FAIL_EXIT( context, priv )              \
BEGIN_MACRO                                     \
    if( (priv) )                                \
    {                                           \
        free_private_data( (priv) );            \
    }                                           \
    if( set_private_data( (context), NULL ) != MI_SUCCESS )    \
    {                                                          \
        syslog( LOG_MAIL | LOG_ERR, "%s(%d) - FATAL ERROR: can't null private data", __FILE__, __LINE__ );\
    }                                           \
    return ActionOnError;                       \
END_MACRO

/* ---------------------------------------------------------------------- */
#define PROLOGUE( context, priv )                                   \
BEGIN_MACRO                                                         \
    (priv) = get_private_data( (context) );                         \
    if( !(priv) )                                                   \
    {                                                               \
        syslog( LOG_MAIL | LOG_ERR, "%s(%d) - FATAL ERROR: can't extract private data from context", __FILE__, __LINE__ );   \
                      /* it's not bug, where priv == NULL */        \
        FAIL_EXIT( (context), (priv) );                             \
    }                                                               \
END_MACRO

/* ---------------------------------------------------------------------- */
int preinit_filter( void )
{
    if( access( ArcDir, W_OK | X_OK ) < 0 )
    {
        wprintLib( "Directory proposed for infected files unaccessible.\n" );
        return -1;
    }
    if( access( ShareDir, W_OK | X_OK ) < 0 )
    {
        wprintLib( "Directory proposed for temporary files unaccessible.\n" );
        return -1;
    }
    if( snprintf( ShareTemplate, sizeof(ShareTemplate)-1,
                  "%s/drweb.tmp.XXXXXX", ShareDir ) < 0 )
    {
        wprintLib( "Path too long for directory proposed for temporary files.\n" );
        return -1;
    }
    if( snprintf( ArcTemplate, sizeof(ArcTemplate)-1,
                  "%s/archive.msg.XXXXXX", ArcDir ) < 0 )
    {
        wprintLib( "Path too long for directory for infected files.\n" );
        return -1;
    }
    return 0;
}

/* ----------------------------------------------------------------------- 
 *  IN: SMFICTX *context;
 *     Context of current filtering session
 *	IN: char *peername;
 *     Host domain name, as determined by a reverse lookup on the host
 *     address.
 *	IN: _SOCK_ADDR *peeraddr;
 *     Host address, as determined by a getpeername call on the SMTP
 *     socket.
 *  NOTE: Invoked on each connection
 * ----------------------------------------------------------------------- */
DWF_retcode_t dwf_connect ( DWF_context_t *context, char* peername, _SOCK_ADDR *peeraddr )
{
    if( peername )
    {
        syslog( LOG_MAIL | LOG_INFO, "connect with %s", peername );
    }
    else
    {
        syslog( LOG_MAIL | LOG_INFO, "connect with unknown" );
    }
    return SMFIS_CONTINUE;
}

/* ----------------------------------------------------------------------- 
 *  IN: SMFICTX *context;
 *     Context of current filtering session
 *	IN: char *helo;
 *     Value passed to HELO/EHLO command, which should be the domain
 *     name of the sending host (but is, in practice, anything the
 *     sending host wants to send).
 *  NOTE: Invoked on SMTP HELO/EHLO command
 * ----------------------------------------------------------------------- */
/*
DWF_retcode_t dwf_helo ( DWF_context_t *context, char* helo )
{
}
*/

/* ----------------------------------------------------------------------- 
 *  IN: SMFICTX *context;
 *     Context of current filtering session
 *	IN: char **from;
 *     Null-terminated SMTP command arguments. from[0] is guaranteed
 *     to be the sender address. Later arguments are the ESMTP
 *     arguments.
 *  NOTE: Invoked on envelope from
 * ----------------------------------------------------------------------- */
DWF_retcode_t dwf_from ( DWF_context_t *context, char* from[] )
{
    DWF_private_data_t  *priv;       
    int                 i;
    
    VERBOSE( syslog( LOG_MAIL | LOG_DEBUG, "envelope from %s", from[0] ) );
    
    create_private_data( context, priv ); /* macro - return if error */
    /* save originator of message */
    if( from && from[0] )
    {
        priv->from = strdup( from[0] ); 
    }
    else
    {   
        priv->from = strdup( "<>" );       
    }
    syslog( LOG_MAIL | LOG_INFO, "Start proccessing from %s", priv->from );
    priv->fname_tmp = strdup( ShareTemplate );
    if( !priv->fname_tmp )
    {
        syslog( LOG_MAIL | LOG_DEBUG, "%s(%d) - no memory for private data", __FILE__, __LINE__ );
        FAIL_EXIT( context, priv );
    }
    priv->fd_tmp = mkstemp( priv->fname_tmp );
    if( priv->fd_tmp < 0 )
    {
        syslog( LOG_MAIL | LOG_DEBUG, "can't create temp file - %m" );
        FAIL_EXIT( context, priv );
    }
    priv->fs_tmp = fdopen( priv->fd_tmp, "w+" );
    if( !priv->fs_tmp )
    {
        syslog( LOG_MAIL | LOG_DEBUG, "can't open stream for temp file - %m" );
        FAIL_EXIT( context, priv );
    }
    if( !priv->from )
    {
        syslog( LOG_MAIL | LOG_DEBUG, "no memory for private data" );
        FAIL_EXIT( context, priv );
    }
    ++priv->from_num;
    fprintf( priv->fs_tmp, "From" );
    for( i = 0; from[i] != NULL; ++i )
    {
        /* ??? ASV: check "%[a-x]" - is valid" */
        fprintf( priv->fs_tmp, " %s", from[i] );
    }
    fprintf( priv->fs_tmp, "\r\n" );
    if( set_private_data( context, priv ) != MI_SUCCESS )
    {
        syslog( LOG_MAIL | LOG_ERR, "%s(%d) - FATAL ERROR: can't store private data", __FILE__, __LINE__ );
        FAIL_EXIT( context, priv );
    }
    return SMFIS_CONTINUE;
}

/* ----------------------------------------------------------------------- 
 *  IN: SMFICTX *context;
 *     Context of current filtering session
 *	IN: char **rcpt;
 *     Null-terminated SMTP command arguments. rcpt[0] is guaranteed
 *     to be the recipient address. Later arguments are the ESMTP
 *     arguments.
 *  NOTE: Invoked on each envelope recipient
 * ----------------------------------------------------------------------- */
DWF_retcode_t dwf_rcpt ( DWF_context_t *context, char* rcpt[] )
{
    int                 i;
    DWF_private_data_t  *priv;
    
    VERBOSE( syslog( LOG_MAIL | LOG_DEBUG, "envelope to %s", rcpt[0] ) );

    PROLOGUE( context, priv );
    fprintf( priv->fs_tmp, "Envelope-To:" );
    for( i = 0; rcpt[i] != NULL; ++i )
    {
            /* ??? ASV: check "%[a-x]" - is valid" */
        fprintf( priv->fs_tmp, " %s", rcpt[i] );
    }
    fprintf( priv->fs_tmp, "\r\n" );
    if( set_private_data( context, priv ) != MI_SUCCESS )
    {
        syslog( LOG_MAIL | LOG_ERR, "%s(%d) - FATAL ERROR: can't store private data", __FILE__, __LINE__ );
        FAIL_EXIT( context, priv );
    }
    return SMFIS_CONTINUE;
}

/* ----------------------------------------------------------------------- 
 *  IN: SMFICTX *context;
 *     Context of current filtering session
 *	IN: char *hdr_name;
 *     Header field name
 *	IN: char *hdr_value;
 *     Header field value
 * NOTE: Invoked on each message header. The content of the header may
 *       have folded white space (that is, multiple lines with
 *       following white space) included.
 * ----------------------------------------------------------------------- */
DWF_retcode_t dwf_header ( DWF_context_t *context, char* hdr_name, char* hdr_value )
{
    DWF_private_data_t  *priv;

    VERBOSE( syslog( LOG_MAIL | LOG_DEBUG, "header - %s:%s", hdr_name, hdr_value ) );
    
    if( !hdr_name )
    {
        return SMFIS_CONTINUE;
    }
    PROLOGUE( context, priv );
    if( strncmp( hdr_name, "Content-Type", strlen("Content-Type") ) == 0 )
    {
        ++priv->contents;
    }
    if( strncmp( hdr_name, "From", strlen("From") ) == 0 )
    {
        ++priv->from_num;
    }
        /* ??? ASV: check "%[a-x]" - is valid" */
    if( hdr_value ) 
    {
        fprintf( priv->fs_tmp, "%s: %s\r\n", hdr_name, hdr_value );
    }
    else
    {
        fprintf( priv->fs_tmp, "%s: <NULL>\r\n", hdr_name );
    }
    if( set_private_data( context, priv ) != MI_SUCCESS )
    {
        syslog( LOG_MAIL | LOG_ERR, "%s(%d) - FATAL ERROR: can't store private data", __FILE__, __LINE__ );
        FAIL_EXIT( context, priv );
    }
    return SMFIS_CONTINUE;
}

/* ----------------------------------------------------------------------- 
 *  IN: SMFICTX *context;
 *     Context of current filtering session
 *  NOTE: Invoked at end of header
 * ----------------------------------------------------------------------- */
DWF_retcode_t dwf_header_end ( DWF_context_t *context )
{
    DWF_private_data_t  *priv;

    VERBOSE( syslog( LOG_MAIL | LOG_DEBUG, "end of header" ) );
    
    PROLOGUE( context, priv );
    fprintf( priv->fs_tmp, "\r\n" );
    if( set_private_data( context, priv ) != MI_SUCCESS )
    {
        syslog( LOG_MAIL | LOG_ERR, "%s(%d) - FATAL ERROR: can't store private data", __FILE__, __LINE__ );
        FAIL_EXIT( context, priv );
    }
    return SMFIS_CONTINUE;
}

/* ----------------------------------------------------------------------- 
 *  IN: SMFICTX *context;
 *     Context of current filtering session
 *	IN: u_char *bodyp;
 *     Pointer to chuck of message body
 *  IN: size_t bodylen;
 *     Length of body data
 *  NOTE: Invoked for each body chunk. There may be multiple body
 *        chunks passed to the filter. End-of-line passed as received
 *        from SMTP
 * ----------------------------------------------------------------------- */
DWF_retcode_t dwf_msg_chunk ( DWF_context_t *context, u_char* chunk, size_t length )  
{
    DWF_private_data_t  *priv;
    
    VERBOSE( syslog( LOG_MAIL | LOG_DEBUG, "message chunk - %d bytes", length ) );
    
    PROLOGUE( context, priv );
    if( fwrite( chunk, length, 1, priv->fs_tmp ) != 1 )
    {
        syslog( LOG_MAIL | LOG_DEBUG, "can't save message body to temp file - %m" );
        FAIL_EXIT( context, priv );
    }
    if( set_private_data( context, priv ) != MI_SUCCESS )
    {
        syslog( LOG_MAIL | LOG_ERR, "%s(%d) - FATAL ERROR: can't store private data", __FILE__, __LINE__ );
        FAIL_EXIT( context, priv );
    }
    return SMFIS_CONTINUE;
}

/* ----------------------------------------------------------------------- 
 *  IN: SMFICTX *context;
 *     Context of current filtering session
 *  NOTE: Invoked at end of message. This routine can perform special
 *        operations such as modifying the message header, body, or envelope.
 * ----------------------------------------------------------------------- */
DWF_retcode_t dwf_msg_end ( DWF_context_t *context )
{
    int                 fd, ret, len, scan_type, i, status, loglen;
    DWF_private_data_t  *priv;
    char                *name = NULL;
    char                NotifyMsg[4096];
    char                ReportMsg[1024];    
    char                *plog;
    int                 *plen;
    
    VERBOSE( syslog( LOG_MAIL | LOG_DEBUG, "end of message" ) );
    
    PROLOGUE( context, priv );
    if( priv->fs_tmp )
    {
        fflush( priv->fs_tmp );
        fclose( priv->fs_tmp );
        priv->fd_tmp = -1;
        priv->fs_tmp = NULL;
    }
    if( bLocal )
    {
        scan_type = DWL_FT_DISKFILE;
    }
    else
    {
        scan_type = DWL_FT_NORMALFILE;
    }
    if( scanOptions & DRWEBD_RETURN_VIRUSES )
    {
        loglen = sizeof(ReportMsg);
        plog = ReportMsg;
        plen = &loglen;
    }
    else
    {
        plog = NULL;
        plen = NULL;
    }
    ret = scanFile( priv->fname_tmp, strlen(priv->fname_tmp),
                    scan_type, scanOptions, &status, plog, plen );
        /* asv - after scan we (currently) dont make tempfail (todo) */
    if( ret != DWL_NOERROR )
    {
        syslog( LOG_MAIL | LOG_ERR, "scan file failed (%s) - possible problem with client or daemon", getLibError( ret )  );
        FAIL_EXIT( context, priv );
    }
    if( status & DERR_VIRUS_MASK )        
    {   /* viruses found */
        syslog( LOG_MAIL | LOG_NOTICE, "in message sended from %s found a virus.\n", priv->from );
        name = strdup( ArcTemplate );
        if( !name )
        {
            syslog( LOG_MAIL | LOG_DEBUG, "%s(%d) - no memory for temp data", __FILE__, __LINE__ );
            FAIL_EXIT( context, priv ); 
        }
        fd = mkstemp( name );
        if( fd < 0 )
        {
            syslog( LOG_MAIL | LOG_DEBUG, "can't create file for store infected mail - %m" );
            free( name );
            FAIL_EXIT( context, priv );
        }
        close( fd );
        if( rename( priv->fname_tmp, name ) < 0 )
        {
            syslog( LOG_MAIL | LOG_DEBUG, "can't move file to archive - %m" );
            free( name );
            FAIL_EXIT( context, priv );
        }

        syslog( LOG_MAIL | LOG_INFO, "infected message stored in archive %s", name );
        
        if( priv->contents > 0 )
        {
            if( smfi_chgheader( context, "Content-Type", 1, "text/plain; charset=us-ascii" ) != MI_SUCCESS )
            {
                syslog( LOG_MAIL | LOG_DEBUG, "can't change content type of mail" );
                free( name );
                FAIL_EXIT( context, priv );
            }
            --priv->contents;
            for( i = 0; i < priv->contents; ++i )
            {
                if( smfi_chgheader( context, "Content-Type", i+2, "" ) != MI_SUCCESS )
                {
                    syslog( LOG_MAIL | LOG_DEBUG, "can't change content type of mail" );
                    free( name );
                    FAIL_EXIT( context, priv );
                }                
            }    
        }
        else
        {
            if( smfi_addheader( context, "Content-Type", "text/plain; charset=us-ascii" ) != MI_SUCCESS )
            {
                syslog( LOG_MAIL | LOG_DEBUG, "can't set content type of mail" );
                free( name );
                FAIL_EXIT( context, priv );
            }
        }
        if( plog )
        {
            len = snprintf( NotifyMsg, sizeof(NotifyMsg),
                            "Message sended from %s infected by virus and was failed to delivery.\r\n"
                            "Diagnostic:\r\n%s\r\n"    
                            "Original message was stored in the archive.\r\n"
                            "Archive record: %s\r\n"
                            "To receive original message please contact postmaster:\r\n%s\r\n\r\n"
                            "\tAntivirus service provided by Dr.Web Daemon\r\n", 
                            priv->from, plog, name, NotifierMail );
        }
        else
        {
            len = snprintf( NotifyMsg, sizeof(NotifyMsg),
                            "Message sent from %s infected by virus and was failed to delivery.\r\n"
                            "Original message was stored in the archive.\r\n"
                            "Archive record: %s\r\n"
                            "To receive original message please contact postmaster:\r\n%s\r\n\r\n"
                            "\tAntivirus service provided by Dr.Web Daemon\r\n", 
                            priv->from, name, NotifierMail );
        }
        free( name );
        if( len < 0 )
        {
            syslog( LOG_MAIL | LOG_DEBUG, "can't create notify message" );
            FAIL_EXIT( context, priv );
        }
        if( smfi_replacebody( context, NotifyMsg, len ) != MI_SUCCESS )
        {
            syslog( LOG_MAIL | LOG_DEBUG, "can't replace message" );
            FAIL_EXIT( context, priv );
        }
        if( bSenderNotify && priv->from && (strlen(priv->from) > 0) )
        {
            if( smfi_addrcpt( context, priv->from ) != MI_SUCCESS )
            {
                syslog( LOG_MAIL | LOG_DEBUG, "can't add recipient" );
                FAIL_EXIT( context, priv );
            }
        }
        if( smfi_chgheader( context, "From", 1, NotifierMail ) != MI_SUCCESS )
        {
            syslog( LOG_MAIL | LOG_DEBUG, "can't add recipient" );
            FAIL_EXIT( context, priv );
        }
        --priv->from_num;
        for( i = 0; i < priv->from_num; ++i )
        {
            if( smfi_chgheader( context, "From", i+2, "" ) != MI_SUCCESS )
            {
                syslog( LOG_MAIL | LOG_DEBUG, "can't change added from of mail" );
                FAIL_EXIT( context, priv );
            }                
        }    
        if( bPostmasterNotify )
        {
            if( smfi_addrcpt( context, PostmasterMail ) != MI_SUCCESS )
            {
                syslog( LOG_MAIL | LOG_DEBUG, "can't add recipient" );
                FAIL_EXIT( context, priv );
            }
        }
    }
    else if( status != DERR_NOERROR )  /* ASV: in future may be cure e-mail */
    {
        syslog( LOG_MAIL | LOG_ERR, "daemon return error (%s) - possible problem with daemon", getDaemonCode( status ) );
        FAIL_EXIT( context, priv );
    } /* else - normal message */
    syslog( LOG_MAIL | LOG_INFO, "Complite proccessing message from %s", priv->from );
    free_private_data( priv );
    if( set_private_data( context, NULL ) != MI_SUCCESS )
    {
        syslog( LOG_MAIL | LOG_ERR, "%s(%d) - FATAL ERROR: can't store private data", __FILE__, __LINE__ );
        return ActionOnError;
    }
    return SMFIS_ACCEPT;
}

/* ----------------------------------------------------------------------- 
 *  IN: SMFICTX *context;
 *     Context of current filtering session
 *  NOTE: Invoked at end of the connection. This is called on close
 *        even if the previous mail transaction was aborted.
 *
 ----------------------------------------------------------------------- */
DWF_retcode_t dwf_close ( DWF_context_t *context )
{
    DWF_private_data_t *priv;
    
    VERBOSE( syslog( LOG_MAIL | LOG_DEBUG, "close session" ) );

    priv = get_private_data( context );
    if( priv )
    {
        free_private_data( priv );
        if( set_private_data( context, NULL ) != MI_SUCCESS )
        {
            syslog( LOG_MAIL | LOG_ERR, "%s(%d) - FATAL ERROR: can't store private data", __FILE__, __LINE__ );
            return ActionOnError;
        }
        return SMFIS_ACCEPT;
    }
    else
    {
        return SMFIS_REJECT;
    }
}


/* ----------------------------------------------------------------------- 
 *  IN: SMFICTX *context;
 *     Context of current filtering session
 *  NOTE: Invoked if message is aborted outside of the control of the
 *        filter, for example, if the SMTP sender issues an RSET
 *        command. If dwf_abort is called, msg_end will not be called
 *        and vice versa.
 * ----------------------------------------------------------------------- */
DWF_retcode_t dwf_abort ( DWF_context_t *context )
{
    DWF_private_data_t *priv;
    
    VERBOSE( syslog( LOG_MAIL | LOG_DEBUG, "abort session" ) );
    
    priv = get_private_data( context );
    if( priv )
    {
        syslog( LOG_MAIL | LOG_INFO, "Abort proccessing message from %s", priv->from );
        free_private_data( priv );
        if( set_private_data( context, NULL ) != MI_SUCCESS )
        {
            syslog( LOG_MAIL | LOG_ERR, "%s(%d) - FATAL ERROR: can't store private data", __FILE__, __LINE__ );
            return ActionOnError;
        }
    }
    return SMFIS_REJECT;
}

#endif /* _DW_FILTER_C */
