/*--------------------------------------------------------------------------
 Demo Dr.Web daemon client for Sendmail Mail Filter API
                                       ID Antivirus Lab http://www.drweb.ru
                                        http://www.drweb.ru/linux-bsd/forum
                                              http://www.drweb.ru/linux-bsd
--------------------------------------------------------------------------*/

#ifndef lint
static const char rcsid[] = "$Id: dwf_main.c,v 1.14 2001/06/13 05:37:48 cvser Exp $";
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <syslog.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <pwd.h>

#include "dwc_lib.h"
#include "dw_filter.h"
#include "dwc_type.h"
#include "dwc_macro.h"

#define DWCLIENT_NOERROR        (0)     /* 0x00000000 */
#define DWCLIENT_EXPAND         (1<<31) /* 0x80000000 */
#define DWCLIENT_DEMO_USAGE     (1<<30) /* 0x40000000 */
#define DWCLIENT_NO_MEMORY      (1<<29) /* 0x20000000 */
#define DWCLIENT_LIB_ERR        (1<<28) /* 0x10000000 */
#define DWCLIENT_SYS_ERR        (1<<27) /* 0x08000000 */
#define DWCLIENT_PARAMETERS     (1<<26) /* 0x04000000 */

static char UserName[ 255 ] = { CHAR0 };
static char PidFile[ PATH_MAX + 1 ] = { CHAR0 };
static char DaemonUnix[ PATH_MAX + 1 ] = { CHAR0 };
static char DaemonInet[ PATH_MAX + 1 ] = { CHAR0 };
static int  DaemonPort = 0;
static char SendmailUnix[ PATH_MAX + 1 ] = { CHAR0 };
static char SendmailInet[ PATH_MAX + 1 ] = { CHAR0 };
static int  SendmailPort = 0;
static int  bDUnix = 0;
static int  bDInet = 0;
static int  bSMUnix = 0;
static int  bSMInet = 0;
static int  bDebug = 0;
static int  Timeout = 0;

static int   parseCmdLine( int argc, char ** argv );
static pid_t daemonize( void );
static int   setuser( void );
static int   usage( void );

/* ----------------------------------------------------------------------- */
int main( int argc, char *argv[] )
{
    int     retcode = DWCLIENT_NOERROR;
    int     libcode = DWL_NOERROR;
    int     dbglevel;
    pid_t   pid;
    char    Buffer[ PATH_MAX + 1 ] = { CHAR0 };

    if( !parseCmdLine( argc, argv ) )
    {
        return usage();
    }
    outputLib( bQuiet, bDebug, TRUE );
    retcode = setuser(); 
    if( retcode != DWCLIENT_NOERROR )
    {        
		wprintLib("Can't set new user.\n");
        return DWCLIENT_LIB_ERR;
    }
    pid = daemonize();
    if( pid > 0 )
    {
        return DWCLIENT_NOERROR;
    }
    else if( pid < 0 )
    {
        wprintLib("Can't daemonize.\n");
        return DWCLIENT_SYS_ERR;
    }        
    if( bDInet )
    {
        libcode = startupLib( DWL_TYPE_TCP_SOCKET, 0, DaemonInet, DaemonPort );
        if( libcode != DWL_NOERROR )
        {
            wprintLib("Can't startup library with tcp/ip transport.\n");
            return DWCLIENT_LIB_ERR;
        }
    }
    else if( bDUnix )
    {
        libcode = startupLib( DWL_TYPE_UNIX_SOCKET, 0, DaemonUnix );
        if( libcode != DWL_NOERROR )
        {
            wprintLib("Can't startup library with unix sockets transport.\n");
            return DWCLIENT_LIB_ERR;
        }
    }
    else if( PidFile[0] != CHAR0 )
    {
        libcode = startupLib( DWL_TYPE_READPIDFILE, 0, PidFile );
        if( libcode != DWL_NOERROR )
        {
            wprintLib("Can't startup library by reading pidfile.\n");
            return DWCLIENT_LIB_ERR;
        }        
    }
    else
    {
        dprintLib("Daemon address not defined. Try use all by defaults.\n");
        libcode = startupLib( DWL_TYPE_DEFAULT, 0 );
        if( libcode != DWL_NOERROR )         
        {
            wprintLib("Can't startup library by.\n");
            return DWCLIENT_LIB_ERR;
        }        
    }
    if( bSMUnix )
    {
        if( snprintf( Buffer, sizeof(Buffer)-1, "local:%s", SendmailUnix ) < 0 )
        {
            wprintLib("Can't format address for init sendmail filter.\n");
            retcode = DWCLIENT_SYS_ERR;
            goto error_exit;
        }
        Buffer[sizeof(Buffer)-1] = CHAR0;
    }
    else /* bSMInet - other alternative absent - it's guaranted by ParseCmdLine */
    {
        if( snprintf( Buffer, sizeof(Buffer)-1, "inet:%d@%s", SendmailPort, SendmailInet ) < 0 )
        {
            wprintLib("Can't format address for init sendmail filter.\n");
            retcode = DWCLIENT_SYS_ERR;
            goto error_exit;
        }
        Buffer[sizeof(Buffer)-1] = CHAR0;
    }
    /* next check access, so we should drop privilegies before execute it */    
    if( preinit_filter() < 0 )
    {
        wprintLib( "Any parameters contain invalid values.\n" );
        goto error_exit;
    }
    dbglevel = (bDebug ? 6 : 0);
    if( smfi_setdbg( dbglevel ) != MI_SUCCESS )
    {
        wprintLib( "Can't set debug level for sendmail filter.\n" );
        retcode = DWCLIENT_SYS_ERR;
        goto error_exit; 
    }
    if( smfi_setconn( Buffer ) != MI_SUCCESS )
    {
        wprintLib( "Can't set address for sendmail filter.\n" );
        retcode = DWCLIENT_SYS_ERR;
        goto error_exit; 
    }
    wprintLib( "connected to %s", Buffer );
    if( (Timeout > 0) && smfi_settimeout( Timeout ) != MI_SUCCESS )
    {
        wprintLib( "Can't set timeout for sendmail filter.\n" );
        retcode = DWCLIENT_SYS_ERR;
        goto error_exit;        
    }
    if( smfi_register( drweb_filter_desc ) != MI_SUCCESS )
    {
        wprintLib( "Can't register filter.\n" );            
        retcode = DWCLIENT_SYS_ERR;
        goto error_exit;        
    }
    wprintLib( "Dr.Web sendmail filter started ..." );
    if( smfi_main() != MI_SUCCESS )
    {
        wprintLib( "Dr.Web sendmail filter returned abnormally ...\n" );            
        retcode = DWCLIENT_SYS_ERR;
    }
    wprintLib( "Dr.Web sendmail filter shutdowned ..." );
  error_exit:
    cleanupLib();
    if( (retcode & ~DERR_MASK ) != 0 ) /* have local errors */
    {
        if( retcode & DWCLIENT_LIB_ERR )
        {
            dprintLib("Results: dwclient library error %d (%s)\n", libcode, getLibError( libcode ) );
        }
    }
    else
    {
        if( retcode != DERR_NOERROR ) /* scan not ok - errors or viruses */
        {
            dprintLib("Results: daemon return code %#x (%s)\n", retcode, getDaemonCode( retcode ) );
        }
    }
    return retcode;
}

/* ----------------------------------------------------------------------- */
static int setuser( void )
{
    struct passwd *account;
    
    if( UserName[0] != CHAR0 )
    {
        account = getpwnam( UserName );
        if( !account )
        {
            wprintLib( "non existen user specified" );
            return DWCLIENT_PARAMETERS;
        }
        if( (setgid(account->pw_gid) < 0)
            ||
            (setuid(account->pw_uid) < 0) )
        {
            wprintLib( "can't change privilegies - %m" );
            return DWCLIENT_SYS_ERR;
        }
    }
    return DWCLIENT_NOERROR;
}
/* ----------------------------------------------------------------------- */
static pid_t daemonize( void )
{
    pid_t          pid;
    int            fd;
    struct rlimit  filelimit;
    
    signal( SIGTTOU, SIG_IGN );
    signal( SIGTTIN, SIG_IGN );
    signal( SIGTSTP, SIG_IGN );
    
    pid = fork();
    if( pid == 0 )
    {
        /* create new session without controling terminal */
        if( setsid() < 0 )
        {
            wprintLib( "can't create new session" );
            return -1;
        }
        /* change current dir to allow umount it */
        if( chdir("/") < 0 )
        {
            wprintLib( "can't change dir to root" );
            return -1;
        }
        umask( 0 );        
        if( getrlimit( RLIMIT_NOFILE, &filelimit ) < 0 )
        {
            wprintLib( "can't get system limits" );
            return -1;
        }
        for( fd = 0; fd < filelimit.rlim_max; ++fd )
        {
            close( fd );
        }
    }
    return pid; 
}

/* ----------------------------------------------------------------------- */
static int parseCmdLine( int argc, char ** argv )
{
    int  intVal, Cont;

    if( argc < 2 )
    {
        wprintLib("\nYou should specify parameters.");
        return 0;
    }
    Cont = 1;
    while( Cont )
    {
        switch( getopt( argc, argv, "+hwqxrlt:m:s:i:d:u:n:p:a:f:o:e:g:b:") )
        {
            case 'e':
                strncpy( NotifierMail, optarg, sizeof(NotifierMail)-1 );
                break;
            case 'g':
                strncpy( PostmasterMail, optarg, sizeof(PostmasterMail)-1 );
                bPostmasterNotify = TRUE;
                break;
            case 'o':
                strncpy( UserName, optarg, sizeof(UserName)-1 );
                break;
            case 'f':
                strncpy( ShareDir, optarg, sizeof(ShareDir)-1 );
                break;
            case 'd':
                strncpy( PidFile, optarg, sizeof(PidFile)-1 );
                break;
            case 'a':
                strncpy( ArcDir, optarg, sizeof(ArcDir)-1 );
                break;
            case 'u':
                strncpy( DaemonUnix, optarg, sizeof(DaemonUnix)-1 );
                bDUnix = 1;
                break;
            case 'm':
                strncpy( SendmailUnix, optarg, sizeof(SendmailUnix)-1 );
                bSMUnix = 1;
                break;
            case 'n':
                strncpy( DaemonInet, optarg, sizeof(DaemonInet)-1 );
                bDInet = 1;
                break;
            case 'i':
                strncpy( SendmailInet, optarg, sizeof(SendmailInet)-1 );
                bSMInet = 1;
                break;
            case 'p':
                intVal = optarg[0];
                if( !isdigit( intVal ) )
                {
                    wprintLib("Error read port argument (-p option).\n");
                    return 0;
                }
                intVal = atoi( optarg );         
                if( intVal >= 0 )
                {
                    DaemonPort = intVal;
                    bDInet = 1;
                }
                break;
            case 's':
                intVal = optarg[0];
                if( !isdigit( intVal ) )
                {
                    wprintLib("Error read port argument (-s option).\n");
                    return 0;
                }
                intVal = atoi( optarg );         
                if( intVal >= 0 )
                {
                    SendmailPort = intVal;
                    bSMInet = 1;
                }
                break;
            case 't':
                intVal = optarg[0];
                if( !isdigit( intVal ) )
                {
                    wprintLib("Error read timeout argument (-t timeout).\n");
                    return 0;
                }
                intVal = atoi( optarg );         
                if( intVal >= 0 )
                {
                    Timeout = intVal / 1000; /* Milter API use seconds */
                }
                break;
            case 'h':
                scanOptions |= DRWEBD_HEURISTIC_ON;
                break;
            case 'b':
                switch( optarg[0] )
                {
                    case 'a':
                        ActionOnError = SMFIS_ACCEPT;
                        break;
                    case 'r':
                        ActionOnError = SMFIS_REJECT;
                        break;
                    case 'd':
                        ActionOnError = SMFIS_DISCARD;
                        break;
                    default:
                        wprintLib("Invalid argument for option -b");
                        return 0;
                }
                break;
            case 'r':     
                scanOptions |= DRWEBD_RETURN_VIRUSES;
                break;
            case 'q':
                bQuiet = 1;
                break;
            case 'w':
                bDebug = 1;
                break;
            case 'l':
                bLocal = 1;
                break;
            case 'x':
                bSenderNotify = 1;
                break;
             case -1 :
                Cont = 0;
                break;
            default :
                if( optarg )
                {
                    wprintLib("\nUnknow option <%c>",optarg[0]);
                }
                return 0;
        }
    }    
    if( NotifierMail[0] == CHAR0 )
    {
        wprintLib("\nYou should specify e-mail address for notifier.");
        return 0;
    }
    if( ArcDir[0] == CHAR0 )
    {
        wprintLib("\nYou should specify directory for infected files.");
        return 0;
    }
    if( ShareDir[0] == CHAR0 )
    {
        wprintLib("\nYou should specify directory for temporary files.");
        return 0;
    }
    if( bDUnix && bDInet )
    {
        wprintLib("\nYou should define only one address for daemon.");
        return 0;
    }                    
    if( ((SendmailInet[0] != CHAR0) && (SendmailPort == 0))
        ||
        ((SendmailInet[0] == CHAR0) && (SendmailPort != 0)) )
    {
        wprintLib("\nYou should define full INET address (name + port) for sendmail.");
        return 0;
    }
    if( (bSMUnix && bSMInet) || (!bSMUnix && !bSMInet) )
    {
        wprintLib("\nYou should define one and only one address for sendmail.");
        return 0;
    }                    
    return 1;
}

/* ----------------------------------------------------------------------- */
static int usage( void )
{
    printf( "\nUsage: drweb-smf [options]"
            "\n"
            "\n -dFILENAME       Dr.Web daemon PID-file (/var/run/drwebd.pid is default)"
            "\n -uSOCKET_FILE    Dr.Web daemon UNIX-socket file"
            "\n -nINETADDRESS    Dr.Web daemon Internet address (IP or domain name)"
            "\n -pPORT           Dr.Web daemon TCP port"
            "\n                  (required -m or combination -i -s)"
            "\n -mSOCKET_FILE    Sendmail filter UNIX-socket file"
            "\n -iINETADDRESS    Sendmail filter Internet address (IP or domain name)"
            "\n -sPORT           Sendmail filter TCP port"
            "\n -tTIMEOUT        timeout (in millisec) for communcations operations"
            "\n -aDIRECTORY      directory with infected messages (required)"
            "\n -l               allow to daemon scan messages storied as local files"
            "\n -fDIRECTORY      directory used for store temp files with messages (required)"
            "\n                  (must be accessible for daemon if -l specified)"
            "\n -h               use heuristic analyser"
            "\n -oUSERNAME       name of user"
            "\n -gMAILADDR       mail (may be local) to that sends notify"
            "\n -eEMAIL          e-mail notifier about viruses,"
            "\n                  public e-mail of postmaster (required)"
            "\n -r               put daemon report to notification message"
            "\n -x               also send notify to sender"
            "\n -bACTION         filter actions on fatal errors:"
            "\n                  ACTION one of { r - reject (default) | a - accept | d - discard }"
            "\n -w               debug mode"
            "\n -q               quiet mode"
            "\n" );
    return DWCLIENT_DEMO_USAGE;
}
