#!/bin/sh

#########################################################
# Settings #
############

# if error allow or not to pass message without checking
#ALLOW_IF_ERROR="Yes"
ALLOW_IF_ERROR="No"

# Name (domain or IP) machine there DrWeb daemon running
#SERVER="localhost"
SERVER="__SERVER__"

# DrWeb daemon port on server 
#PORT="3000"
PORT="__PORT__"

# E-Mail from that daemon writes reports
#DAEMON_EMAIL="DrWeb-DAEMON@your.domain"
DAEMON_EMAIL="__NAME__@__DOMAIN__"

# E-Mail that placed to reports 
#POSTMASTER_EMAIL="postmaster@your.domain"
POSTMASTER_EMAIL="__NAME__@__DOMAIN__"

# Mail address for alerts and problems reports (can be local)
#ADMIN_EMAIL="sergey"
ADMIN_EMAIL="__MAIL__"

# Directory for save message before check
#TMP_DIR=/var/spool/drweb
TMP_DIR=__DIRECTORY__

# Directory for infected messages
#INFECTED_DIR=/opt/drweb/infected.!!!
INFECTED_DIR=__DIRECTORY__

# Directory where DrWeb installed
#DRWEB_DIR=/opt/drweb
DRWEB_DIR=__DIRECTORY__

#########################################################
# Variables and constants

# Command for postfix
SENDMAIL_CMD="/usr/sbin/sendmail"

# Command for client to daemon
SCANNER_CMD="$DRWEB_DIR/clients/drwebdc/drwebdc -n"$SERVER" -p"$PORT" -q -f"

# Exit codes from <sysexits.h>
EX_OK=0
EX_TEMPFAIL=75
EX_UNAVAILABLE=69

# Get parameters
SENDER=$1
RECEIVER=$2

# Start code

# Clean up when done or when aborting.
trap "rm -f $TEMP_FILE" 0 1 2 3 15

# Create tempfile
TEMP_FILE=`mktemp -q $TMP_DIR/msg.XXXXXX`

# Start processing.
cd $TMP_DIR || { echo $TMP_DIR does not exist; exit $EX_TEMPFAIL; }

# Save to tempfile.
cat > $TEMP_FILE || { echo Cannot save mail to file; exit $EX_TEMPFAIL; }

# Call Dr.Web.
$SCANNER_CMD $TEMP_FILE
EXIT_STATUS=$?
if [ $EXIT_STATUS -eq 0 ]
then
    # Send normal message
    $SENDMAIL_CMD -bm -f $SENDER < $TEMP_FILE
    exit $?
elif [ $EXIT_STATUS -eq 32 -o $EXIT_STATUS -eq 64 -o $EXIT_STATUS -eq 128 ] 
then
    # Move infected message to archive
    mv -f $TEMP_FILE $INFECTED_DIR    
    
    # Generate message to sender
    TEMP_FILE=`mktemp -q $TMP_DIR/drweb.XXXXXX`
    printf "From: "$DAEMON_EMAIL"\nTo: "$SENDER"\nSubject: Mail delivery failed (virus detected)\nYour message sended to "$RECEIVER" failed to delivery.\nReason: virus detected.\nOriginal message stored in archive.\nTo receive original, please send request on "$POSTMASTER_EMAIL"\n---\n   Antivirus service provided by Dr.Web Daemon\n.\n" >> $TEMP_FILE
	    
    # Send virus alert
    $SENDMAIL_CMD -bm -f $DAEMON_EMAIL < $TEMP_FILE
    rm -f $TEMP_FILE

    # Generate message to admin
    TEMP_FILE=`mktemp -q $TMP_DIR/drweb.XXXXXX`
    
    printf "From: "$DAEMON_EMAIL"\nTo: "$ADMIN_EMAIL"\nSubject: Mail delivery failed (virus detected)\nDear Postmaster,\n Message sended by "$SENDER"\n to "$RECEIVER" failed to delivery.\nReason: virus detected.\n---\n   Antivirus service provided by Dr.Web Daemon\n.\n" >> $TEMP_FILE 
	    
    # Send report to admin
    $SENDMAIL_CMD -bm -f $DAEMON_EMAIL < $TEMP_FILE
    rm -f $TEMP_FILE
    
    exit $EX_OK
else
    # Any problems with daemon or client
    if [ $ALLOW_IF_ERROR = "Yes" ]
    then
        $SENDMAIL_CMD -bm -f $SENDER < $TEMP_FILE
    fi
    rm -f $TEMP_FILE
    
    # Generate report
    TEMP_FILE=`mktemp -q $TMP_DIR/drweb.XXXXXX`
    
    if [ $EXIT_STATUS -le 65535 ]
    then
        printf "From: "$DAEMON_EMAIL"\nTo: "$ADMIN_EMAIL"\nSubject: Antivirus Software Fail\nHello,\nMessage sended by from "$SENDER" to "$RECEIVER" failed to scan.\nDiagnostic: possible problems with daemon (error code "$EXIT_STATUS")\n---\n   Antivirus service provided by Dr.Web Daemon\n.\n" >> $TEMP_FILE
    else
        printf "From: "$DAEMON_EMAIL"\nTo: "$ADMIN_EMAIL"\nSubject: Antivirus Software Fail\nHello,\nMessage sended by from "$SENDER" to "$RECEIVER" failed to scan.\nDiagnostic: possible problems with client (error code "$EXIT_STATUS")\n---\n   Antivirus service provided by Dr.Web Daemon\n.\n" >> $TEMP_FILE
    fi	
	    
    # Send report to admin
    $SENDMAIL_CMD -bm -f $DAEMON_EMAIL < $TEMP_FILE
    rm -f $TEMP_FILE
    
    if [ $ALLOW_IF_ERROR = "Yes" ]
    then
        exit $EX_OK
    else
        exit $EX_TEMPFAIL   
    fi	
fi

